--[[
Raka's Outstanding Spell Announcer
]]

local ADDON_NAME, ns = ...
local RSA_DebugMode = false	-- true to debug(always available, print all announces to default chat frame only)

if not RSA_Data then
	RSA_Data = {}
	RSA_Data.Enabled = true
	RSA_Data.InstanceOnly = true
end

-- Core Function --
local playername = UnitName('player')
local function intrHandler(self, event)
	if not RSA_DebugMode then
		if RSA_Data.InstanceOnly and not IsInInstance() then return end
	end
	
	if not RSA_Data.Enabled then return end

	local _, spelltype, _, _, sourceName, sourceFlags, sourceRaidFlags, _, destName, destFlags, destRaidFlags, spellid, spellname, _, extraspellid, extraspellname = CombatLogGetCurrentEventInfo()
	
	local linkformat = '|cff71d5ff|Hspell:%d:0|h[%s]|h|r'
	local msg = ''
	local sourceMarker = self.RSA_GetRaidMarker(sourceRaidFlags, sourceName)
	local destMarker = self.RSA_GetRaidMarker(destRaidFlags, destName)
	
	-- Only Announce Events Fired by Friendly Player -- 
	if bit.band(sourceFlags, COMBATLOG_OBJECT_TYPE_PLAYER) > 0 and bit.band(sourceFlags, COMBATLOG_OBJECT_REACTION_FRIENDLY) > 0 then
		-- Player Interrupt and Dispel --
        if RSA_DebugMode then
            print(spellname..' '..spellid..' '..spelltype)
        end
        
		if (spelltype == 'SPELL_INTERRUPT' or spelltype == 'SPELL_DISPEL') then
			
			local _, spellcd, _ = GetSpellCooldown(spellname)
			local spelllink = string.format(linkformat, spellid, spellname)
			local exspelllink = string.format(linkformat, extraspellid, extraspellname)
			
			if spelltype == 'SPELL_INTERRUPT' then
				if sourceName == playername then
					msg = string.format(ns.L['Interrupt'] .. ' [%s] -> %s! (%s, CD:%ds) ', destMarker, exspelllink, spelllink, spellcd)
				else
					msg = string.format(ns.L['Interrupt'] .. ' [%s] -> %s! (%s, %s) ', destMarker, exspelllink, spelllink, sourceMarker)
				end
			end
			
			if spelltype == 'SPELL_DISPEL' and bit.band(destFlags, COMBATLOG_OBJECT_REACTION_FRIENDLY) == 0 then
				if sourceName == playername and spellcd > 0 then
					msg = string.format(ns.L['Dispel'] .. ' [%s] -> %s! (%s, CD:%ds) ', destMarker, exspelllink, spelllink, spellcd)
				elseif sourceName ~= playername then
					msg = string.format(ns.L['Dispel'] .. ' [%s] -> %s! (%s, %s) ', destMarker, exspelllink, spelllink, sourceMarker)
				else
					msg = string.format(ns.L['Dispel'] .. ' [%s] -> %s! (%s) ', destMarker, exspelllink, spelllink)
				end
			end
			
		end
		
		-- CC Broken --
		if spelltype == 'SPELL_AURA_BROKEN' and ns.CCSpells[spellid] then
			local spelllink = string.format(linkformat, spellid, spellname)
		
			if sourceName == playername then
				print(string.format(ns.L['|cFFFF5151ROSA|r: Oops... You broke CC %s with melee attack...'], spelllink))
			else
				msg = string.format(ns.L['CC Broken'] .. ' [%s] -> %s! (' .. L['melee'] ..' , %s)', destMarker, spelllink, sourceMarker)
			end
		elseif spelltype == 'SPELL_AURA_BROKEN_SPELL' and ns.CCSpells[spellid] then
			local spelllink = string.format(linkformat, spellid, spellname)
			local exspelllink = string.format(linkformat, extraspellid, extraspellname)
		
			if sourceName == playername then
				print(string.format(ns.L['|cFFFF5151ROSA|r: Oops... You broke CC %s with %s...'], spelllink, exspelllink))
			else
				msg = string.format(ns.L['CC Broken'] .. ' [%s] -> %s! (%s, %s)', destMarker, spelllink, exspelllink, sourceMarker)
			end
		end
		
		-- Bloodlusts --
		if spelltype == 'SPELL_AURA_APPLIED'then
			if ns.Bloodlusts[spellid] and destName == playername then -- only when player receives buff
				local spelllink = string.format(linkformat, spellid, spellname)
				msg = string.format(ns.L['Burst'] .. ' [%s] -> %s! ' .. ns.L['Rush'] .. '!', sourceMarker, spelllink)
			end
		end
		
		-- Combat Revives and Miscellaneous --
		if spelltype == 'SPELL_CAST_SUCCESS' then 
			if ns.CombatRevives[spellid] then
				local spelllink = string.format(linkformat, spellid, spellname)
				msg = string.format(ns.L['Revive'] .. ' [%s] -> %s! (%s)', sourceMarker, destMarker, spelllink)
			elseif ns.Miscellaneous[spellid] then
				if destName and sourceName ~= destName then
					msg = string.format(ns.Miscellaneous[spellid] .. ' [%s] -> [%s]!', sourceMarker, destMarker)
				else
					msg = string.format(ns.Miscellaneous[spellid] .. ' -> [%s]!', sourceMarker)
				end
			elseif spellname:match(ns.L['^Portal:']) then
			-- Mage Portals
				local destination = spellname:gsub(ns.L['^Portal:'], '')
				msg = string.format(ns.L['Portal'] .. ' -> [%s]! (%s)', destination:gsub('%s+', ''), sourceMarker)
			elseif spellname:match(ns.L['^Ancient Portal:']) then
			-- Mage being a jerk
				local destination = spellname:gsub(ns.L['^Portal:'], '')
				msg = string.format(ns.L['Warning: Ancient Portal!'] .. ' -> [%s]! (%s)', destination:gsub('%s+', ''), sourceMarker)
			elseif ns.Bewares[spellid] then
			-- Bewares
				local spelllink = string.format(linkformat, spellid, spellname)
				if not destName or sourceName == destName then
					msg = string.format(ns.L['Beware!'] .. ' [%s] -> [%s]!', spelllink, sourceMarker)
				else
					msg = string.format(ns.L['Beware!'] .. ' [%s] -> [%s]! (%s)', spelllink, destMarker, sourceMarker)
				end
			end
		end
		
		-- Aura Stolen --
		if spelltype == 'SPELL_STOLEN' then
			local exspelllink = string.format(linkformat, extraspellid, extraspellname)
			msg = string.format(ns.L['Aura Stolen'] .. '[%s] %s -> [%s]!', destMarker , exspelllink, sourceMarker)
		end  

		if RSA_Data.Enabled then
			self.RSA_SendChatMessage(msg)
		end
	end
end

local f = CreateFrame('Frame', nil, UIParent)
if RSA_Data.Enabled then
	f:RegisterEvent('COMBAT_LOG_EVENT_UNFILTERED')
end
f:SetScript('OnEvent', intrHandler)

f.RSA_GetRaidMarker = function(unitFlags, unitName)
	if not unitFlags then return end
	if not unitName then unitName = '' end
	local marker =	(bit.band(unitFlags, COMBATLOG_OBJECT_RAIDTARGET1) > 0 and '{rt1}') or 
					(bit.band(unitFlags, COMBATLOG_OBJECT_RAIDTARGET2) > 0 and '{rt2}') or
					(bit.band(unitFlags, COMBATLOG_OBJECT_RAIDTARGET3) > 0 and '{rt3}') or
					(bit.band(unitFlags, COMBATLOG_OBJECT_RAIDTARGET4) > 0 and '{rt4}') or
					(bit.band(unitFlags, COMBATLOG_OBJECT_RAIDTARGET5) > 0 and '{rt5}') or
					(bit.band(unitFlags, COMBATLOG_OBJECT_RAIDTARGET6) > 0 and '{rt6}') or
					(bit.band(unitFlags, COMBATLOG_OBJECT_RAIDTARGET7) > 0 and '{rt7}') or
					(bit.band(unitFlags, COMBATLOG_OBJECT_RAIDTARGET8) > 0 and '{rt8}') or ''
	return marker .. unitName
end

f.RSA_SendChatMessage = function(msg)
	if msg and string.len(msg) > 0 then
		_, instanceType = IsInInstance()
		if not RSA_DebugMode then
			if IsInGroup(LE_PARTY_CATEGORY_INSTANCE) or IsInRaid(LE_PARTY_CATEGORY_INSTANCE) or instanceType == 'pvp' or instanceType == 'arena' then
				ChatThrottleLib:SendChatMessage('ALERT', 'RSA', msg, 'INSTANCE_CHAT')
			elseif IsInRaid() then
				ChatThrottleLib:SendChatMessage('ALERT', 'RSA', msg, 'RAID')
			elseif IsInGroup() then
				ChatThrottleLib:SendChatMessage('ALERT', 'RSA', msg, 'PARTY')
			else
				ChatThrottleLib:SendChatMessage('ALERT', 'RSA', msg, 'SAY')
			end
		else
			print(msg)
		end
	end
end

SLASH_ROSA1 = '/rosa'
local function aphandler(msg)
	msg = string.lower(msg)
	if msg:match('^disable') then
		f:UnregisterAllEvents()
		RSA_Data.Enabled = false
		print('|cFFFF5151ROSA|r |cFFFF0000' .. ns.L['Disabled'] .. '|r.')
	elseif msg:match('^enable') then
		f:RegisterEvent('COMBAT_LOG_EVENT_UNFILTERED')
		RSA_Data.Enabled = true
		print('|cFFFF5151ROSA|r |cFF00FF00' .. ns.L['Enabled'] .. '|r.')
	elseif msg:match('^io') then
		if RSA_Data.InstanceOnly then
			RSA_Data.InstanceOnly = false
			print('|cFFFF5151ROSA|r ' .. ns.L['Instance Only Mode'] .. ' |cFFFF0000' .. ns.L['Disabled'] .. '|r.')
		else
			RSA_Data.InstanceOnly = true
			print('|cFFFF5151ROSA|r ' .. ns.L['Instance Only Mode'] .. ' |cFF00FF00' .. ns.L['Enabled'] .. '|r.')
		end
	elseif msg:match('^help') then
		print('====|cFFFF5151ROSA|r HELP====\n' .. ns.L['/rosa - ROSA Status.\n/rosa enable/disable - Enable/Disable ROSA.\n/rosa io - Toggle Instance Only Mode.\n/rosa help - This help.'] )
	else
		local mode = (RSA_Data.InstanceOnly and (' |cFF00FF00' .. ns.L['Enabled'] .. '|r.')) or (' |cFFFF0000' .. ns.L['Disabled'] .. '|r.')
		if RSA_Data.Enabled then
			print('|cFFFF5151ROSA|r ' .. ns.L['is currently'] .. ' |cFF00FF00' .. ns.L['Enabled'] .. '|r. '.. ns.L['Instance Only Mode'] .. mode .. ns.L['Type "/rosa help" for more info.'])
		else
			print('|cFFFF5151ROSA|r ' .. ns.L['is currently'] .. ' |cFFFF0000' .. ns.L['Disabled'] .. '|r. ' .. ns.L['Instance Only Mode'] .. mode .. ns.L['Type "/rosa help" for more info.'])
		end
	end
end
SlashCmdList['ROSA'] = aphandler